/**
 * Gulp config file
 * Copyright © 2017 Alexander Konov. All rights reserved.
 */

'use strict';

var gulp = require('gulp');

var browserify = require('gulp-browserify');
var concat = require('gulp-concat');
var sass = require('gulp-sass');
var concatCss = require('gulp-concat-css');
var autoPreFixer = require('gulp-autoprefixer');
var uglify = require('gulp-uglify');
var cleanCSS = require('gulp-clean-css');


var log = function (msg) {
    console.log(msg + "\n");
};

gulp.task('test', function () {
    var Server = require('karma').Server;
    return new Server({
        configFile: __dirname + '/karma.conf.js',
        singleRun: true
    }).start();
});

gulp.task('dev', ['connect'], function () {
    var jsTasks, cssTasks;

    (jsTasks = function () {
        gulp.src('./src/main.coffee', {read: false})
            .pipe(browserify({
                transform: ['coffeeify'],
                debug: true,
                extensions: ['.coffee', '.js'],
            }))
            .pipe(concat('jquery-attachments.js'))
            .pipe(gulp.dest('./dev'));
        log('JS compiled.');
    })();
    gulp.watch('./src/**/*.coffee', jsTasks);

    (cssTasks = function () {
        gulp.src('./src/**/*.sass')
            .pipe(sass().on('error', sass.logError))
            .pipe(autoPreFixer())
            .pipe(concatCss('jquery-attachments.css'))
            .pipe(gulp.dest('./dev'));
        log('CSS compiled.');
    })();
    gulp.watch('./src/**/*.sass', cssTasks);

});

gulp.task('dist', function () {

    gulp.src('./src/main.coffee', {read: false})
        .pipe(browserify({
            transform: ['coffeeify'],
            extensions: ['.coffee', '.js'],
        }))
        .pipe(concat('jquery-attachments.min.js'))
        .pipe(uglify())
        .pipe(gulp.dest('./dist'));

    gulp.src('./src/**/*.sass')
        .pipe(sass().on('error', sass.logError))
        .pipe(autoPreFixer())
        .pipe(concatCss('jquery-attachments.min.css'))
        .pipe(cleanCSS())
        .pipe(gulp.dest('./dist'));

});

gulp.task('connect', function () {
    var connect = require('gulp-connect');
    connect.server({
        root: './',
        livereload: false
    });
});
