###
# Collection for files (file model)
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

class FilesCollection

  ###
  # Constructor
  # @param DI di - dependencies object
  ###
  constructor: (di) ->
    di.resolve ['utils.DataMap', 'models.File']

    @_di = di
    @_map = di.createObject 'utils.DataMap'
    return

  ###
  # Add files to collection from array
  # @param File[]|Object[] files - array with files or array objects with fields of file
  ###
  addFiles: (files) ->
    return if !files or !files.length
    for file in files
      @addFile file
    return

  ###
  # Add file to collection
  # @param File|Object file - adding file or object with fields of file
  ###
  addFile: (file) ->
    file = @_objectToFile file unless file instanceof @_di.get 'models.File'
    return false unless file
    @_map.add file.getField('id'), file
    return

  ###
  # Check exists file in collection by id
  # @param int|string id - file id
  # @return bool
  ###
  has: (id) ->
    @_map.has id

  ###
  # Get file from collection
  # @param string|int id - file id
  # @return File|null
  ###
  getFile: (id) ->
    @_map.get id

  ###
  # Get all files from collection
  # @return File[]
  ###
  getFiles: ->
    @_map.toArray()

  ###
  # Remove file from collection
  # @param string|int id - file id
  ###
  removeFile: (id)->
    @_map.remove id
    return

  ###
  # Remove all files in collection
  ###
  clear: ->
    @_map.clear()
    return

  ###
  # Number of files in collection
  # @return int
  ###
  length:->
    @_map.length()

  ###
  # Create File object from objects with file fields
  # @return File
  ###
  _objectToFile: (fields)->
    @_di.createObject 'models.File', [fields]

module.exports = FilesCollection
