###
# File model
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

class File

  ###
  # Constructor
  # @param null|mixed{} data - object with data fields
  # @see File::setFields for more information
  ###
  constructor: (data) ->
    @_fields =
      id: null
      name: null
      extension: null
      size: null
      url: null
      preview_url: null
      created_at: null
      updated_at: null
    @setFields data if data
    return

  ###
  # Set few fields
  # @param null|mixed{} data - map object with data fields
  # Fields:
  #   int|string id - file ID
  #   string name - file name
  #   string extension - file extension
  #   int size - file size in bytes
  #   string preview_url - image preview url for this file
  #   int created_at - file created date (! save on server DB !)
  #   int updated_at - file updated date
  # Example
  #   file = new File {id:1, name: 'file'}
  ###
  setFields: (fields) ->
    for own name, value of fields
      @setField name, value
    return

  ###
  # Set field value
  # @param string name
  # @param mixed value
  # @see File::setFields for information about data types value
  ###
  setField: (name, value) ->
    @_fields[name] = String value if @_fields[name] isnt undefined
    return

  ###
  # Return all fields
  # @return mixed{} - object with data fields
  # @see File::setFields for information about data types fields
  ###
  getFields: () ->
    if Object.assign
      return Object.assign {}, @_fields
    data = {}
    for own name, value of @_fields
      data[name] = value
    data

  ###
  # Return field value by name
  # @param string name - field name
  # @return mixed - field value
  # @see File::setFields for information about data type field
  ###
  getField: (fieldName) ->
    if @_fields[fieldName] isnt null then @_fields[fieldName] else null

module.exports = File