###
# DataMap util
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict';

class DataMap

  ###
  # Constructor
  # @param null|mixed{}|mixed[] items - items for map
  # @see DataMap::add for more information
  ###
  constructor: (items) ->
    @_items = {}
    @_order = []
    @addItems items if items
    return

  ###
  # Checking of existence item by key in map
  # @param string|int key - key of item
  # @return bool
  ###
  has: (key) ->
    @_items[key] isnt undefined

  ###
  # Add few item to map
  # @param null|mixed{}|mixed[] items - items for map
  # @see DataMap::add for more information
  ###
  addItems: (items) ->
    for own key of items
      @add key, items[key]
    return

  ###
  # Add item to map
  # @param string|int key - item key
  # @param mixed - item value
  ###
  add: (key, value) ->
    if @has key
      @_items[key] = value
    else
      @_items[key] = value
      @_order.push key.toString()
    return

  ###
  # Get item value by key
  # @param string|int key - item key
  # @return mixed|null
  ###
  get: (key) ->
    if @has key then @_items[key] else null

  ###
  # Remove item in map by key
  # @param string|int key - item key
  ###
  remove: (key) ->
    delete @_items[key]
    orderIndex = @_order.indexOf key.toString()
    if orderIndex isnt -1
      @_order.splice orderIndex, 1
    return

  ###
  # Call function for each item in map
  # @param callable fn(key, value) - function calls for each item
  ###
  each: (fn) ->
    for key in @_order
      fn key, @get key
    return

  ###
  # Return array with all added items to map
  # Items sorted by order adding to map
  # @return {key: key, value: value}[]
  ###
  all: ->
    result = []
    for key in @_order
      result.push {key: key, value: @get key}
    result

  ###
  # Return array with all added values to map
  # Items sorted by order adding to map
  # @return mixed[]
  ###
  toArray: ->
    result = []
    for key in @_order
      result.push @get key
    result

  ###
  # Remove all items in map
  ###
  clear: ->
    @_items = {}
    @_order = []
    return

  ###
  # Number of items in map
  # @return int
  ###
  length: ->
    @_order.length

module.exports = DataMap
