###
# Translate
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

###
# Escape string for regular expression
# @param string str - non-escaped string
# @return string
###
escape = (str) ->
  str.replace /[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&"

class Translate

  ###
  # Constructor
  # @param string{} phrases - object with translate dictionary,
  # where key - phrase in its original language, value - translated phrase
  # Example:
  #   {
  #       'Hello, {name}!': 'Привет, {name}!',
  #       ...
  #   }
  ###
  constructor: (phrases)->
    @_phrases = phrases

  ###
  # Translate phrase with params
  # @param string phrase - phrase in its original language
  # @param string{} params - object with params for replace, where key should be consist in phrase how {paramKey}
  # @return string
  # if phrase not found in dictionary, when only replaced params.
  ###
  t: (phrase, params) ->
    translate = if @_phrases[phrase] then @_phrases[phrase] else phrase
    translate = @_replaceParams translate, params if params
    translate

  ###
  # Replace params in phrase to params value
  # @param string phrase - original phrase
  # @param string{} params - object with params for replace
  # @return string
  ###
  _replaceParams: (phrase, params) ->
    for own key, value of params
      expr = new RegExp escape('{' + key + '}'), 'g'
      phrase = phrase.replace expr, value
    phrase

module.exports = Translate