##
# Template for view main window for select and manage files
# Copyright © 2017 Alexander Konov. All rights reserved.
##
# @var Translate translate - object for getting of translated text
# @var string uid - unique window id
# @var string accept - accept extensions for upload
##
# Required attributes:
#  [jq-attachments-window] - main container for window
#  [jq-attachments-window-close] -  attribute of element which had to close window by click (click event)
#  [uid="{FILE_ID}"] - unique window id. Should be have value = uid and be attribute for main container
#  [jq-attachments-upload] - input field (type="file") for upload
#  [jq-attachments-upload-send] -  attribute of element that is a button for start upload (by click event)
#  [jq-attachments-progress] -  attribute of element which show loading process
#  [jq-attachments-files] -  attribute of element which contain loaded files
#  [jq-attachments-files-more] -  attribute of element that is a button for load more files from server (by click event)
#  [jq-attachments-apply] -  attribute of element that is a button for apply select files
##

'use strict'

template = """
<section uid="<%= uid%>" class="jq-attachments-window" jq-attachments-window>
    <div class="jq-attachments-window-content">
        <header class="jq-attachments-window-content-header">
            <form enctype="multipart/form-data">
                <div class="jq-attachments-upload">
                  <label>
                    <input type="file" name="file[]" jq-attachments-upload multiple accept="<%= accept%>"/>
                    <span data-role="upload-text"><%= translate.t('Select files')%></span>
                  </label>
                </div>
                <input type="submit" class="jq-attachments-upload-send" value="upload" jq-attachments-upload-send disabled/>
            </form>
            <div class="jq-attachments-progress" title="<%= translate.t('Wait...')%>" jq-attachments-progress>
                <div></div>
            </div>
            <span class="jq-attachments-window-close" jq-attachments-window-close>+</span>
        </header>
        <main class="jq-attachments-window-content-main">
            <div class="jq-attachments-files">
              <div jq-attachments-files>
              </div>
              <input type="submit" value="<%= translate.t('Load more files')%>" class="jq-attachments-files-more" jq-attachments-files-more/>
            </div>
        </main>
        <footer class="jq-attachments-window-content-footer">
            <input type="submit" value="<%= translate.t('Save')%>" class="jq-attachments-apply" jq-attachments-apply/>
            <input type="submit" value="<%= translate.t('Cancel')%>" class="jq-attachments-cancel" jq-attachments-window-close/>
        </footer>
    </div>
    <div class="jq-attachments-window-bg" jq-attachments-window-close></div>
</section>

<%
//script for  UI logic (custom upload form)
var interval = null;
var renderUpload = function ()
 {
   var fieldFile = document.querySelector('[uid="' + uid + '"] [jq-attachments-upload]');
   if (fieldFile) {
     clearInterval(interval);
     var uploadSendButton = document.querySelector('[uid="' + uid + '"] [jq-attachments-upload-send]');
     var uploadText = document.querySelector('[uid="' + uid + '"] [data-role="upload-text"]');
     fieldFile.onchange = function () {
        var countFiles = this.files.length;
        var message;
        if (countFiles) {
          message = translate.t('Selected {countFiles} files', {countFiles: countFiles});
        } else {
          message = translate.t('Select files');
        }
        uploadText.innerHTML = message;
        uploadSendButton.disabled = !countFiles;
     };
   }
};

interval = setInterval(renderUpload, 100);
%>
"""
module.exports = template