###
# Test for file collection
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

DI = require '../../src/di/DI'
FilesCollection = require '../../src/collections/FilesCollection'

class TestFile
  constructor: (data) ->
  getField: (fieldName) ->


describe 'collections.FilesCollection', ->
  createFile = null
  createMap = null
  di = null
  map = null

  beforeEach ->
    createFile = (data) ->
      file = new TestFile(data)
      spyOn file, 'getField'
      file

    createMap = () ->
      jasmine.createSpyObj 'DataMap', [
        'add', 'has', 'get', 'toArray',
        'remove', 'clear', 'length'
      ]

    map = createMap()
    di = new DI()
    di
      .set 'models.File', TestFile
      .set 'utils.DataMap', () -> map
    return

  ######################################################################################################################

  it 'should be exist resolved requires dependencies', ->
    expect(-> new FilesCollection di).not.toThrow()
    return

  ######################################################################################################################

  it 'should be add file from File object', ->
    file = createFile()
    file.getField.and.callFake (key) -> 123

    collection = new FilesCollection di
    collection.addFile file

    expect(file.getField).toHaveBeenCalledWith 'id'
    expect(map.add).toHaveBeenCalledWith 123, file
    return

  ######################################################################################################################

  it 'should be add files from File objects', ->
    files = [createFile(), createFile(), createFile()]
    files[0].getField.and.callFake (key) -> 1
    files[1].getField.and.callFake (key) -> 2
    files[2].getField.and.callFake (key) -> 3

    collection = new FilesCollection di
    collection.addFiles files

    expect(files[0].getField).toHaveBeenCalledWith 'id'
    expect(map.add).toHaveBeenCalledWith 1, files[0]

    expect(files[1].getField).toHaveBeenCalledWith 'id'
    expect(map.add).toHaveBeenCalledWith 2, files[1]

    expect(files[2].getField).toHaveBeenCalledWith 'id'
    expect(map.add).toHaveBeenCalledWith 3, files[2]
    return

  ######################################################################################################################

  it 'should be add file from object with File fields', ->
    file = {id: 1, name: 'name'}

    spyOn(di, 'createObject')
    collection = new FilesCollection di
    collection.addFile file

    expect(di.createObject).toHaveBeenCalledWith 'models.File', [file]
    return

  ######################################################################################################################

  it 'should be add files from objects with File fields', ->
    files = [
      {id: 1, name: 'name'}
      {id: 2, name: 'name2'}
      {id: 3, name: 'name3'}
    ]

    spyOn(di, 'createObject')
    collection = new FilesCollection di
    collection.addFiles files

    expect(di.createObject).toHaveBeenCalledWith 'models.File', [files[0]]
    expect(di.createObject).toHaveBeenCalledWith 'models.File', [files[1]]
    expect(di.createObject).toHaveBeenCalledWith 'models.File', [files[2]]
    return

  ######################################################################################################################

  it 'should be check the existence of a file in collection', ->
    file = createFile()

    map.has.and.callFake (key) -> true
    collection = new FilesCollection di
    collection.addFile file

    expect(collection.has(1)).toBeTruthy()
    expect(map.has).toHaveBeenCalledWith 1

    map.has.and.callFake (key) -> false
    collection = new FilesCollection di
    collection.addFile file

    expect(collection.has(1)).toBeFalsy()
    expect(map.has).toHaveBeenCalledWith 1
    return

  ######################################################################################################################

  it 'should be get file from collection', ->
    file = createFile()

    map.get.and.callFake (key) -> file
    collection = new FilesCollection di
    collection.addFile file

    expect(collection.getFile(1)).toEqual file
    expect(map.get).toHaveBeenCalledWith 1
    return

  ######################################################################################################################

  it 'should be get files from collection', ->
    files = [createFile(), createFile(), createFile()]

    map.toArray.and.callFake (key) -> files
    collection = new FilesCollection di
    collection.addFiles files

    expect(collection.getFiles()).toEqual files
    expect(map.toArray).toHaveBeenCalled()
    return

  ######################################################################################################################

  it 'should be remove file from collection', ->
    file = createFile()

    collection = new FilesCollection di
    collection.addFile file
    collection.removeFile 1

    expect(map.remove).toHaveBeenCalledWith 1
    return

  ######################################################################################################################

  it 'should be clear collection', ->
    collection = new FilesCollection di
    collection.clear()

    expect(map.clear).toHaveBeenCalled()
    return

  ######################################################################################################################

  it 'should be get size collection', ->
    map.length.and.callFake ()-> 10
    collection = new FilesCollection di

    expect(collection.length()).toBe(10)
    expect(map.length).toHaveBeenCalled()
    return
