###
# Test for dependency injection
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

DI = require '../../src/di/DI'

class TestClassA
  constructor: (@a, @b, @c) ->

  getA: ->
    @a

  getB: ->
    @b

  getC: ->
    @c
class TestClassB extends TestClassA
class TestClassC extends TestClassA


describe 'di.DI', ->
  di = null

  beforeEach ->
    di = new DI()

  ######################################################################################################################

  it 'should be set dependency', ->
    di
      .set 'TestClassA', TestClassA
      .set 'TestClassB', TestClassB
      .set 'TestClassC', TestClassC

    expect(di.get('TestClassA')).toBe(TestClassA)
    expect(di.get('TestClassB')).toBe(TestClassB)
    expect(di.get('TestClassC')).toBe(TestClassC)
    return

  ######################################################################################################################

  it 'should be set throw error, if dependency is not resolved', ->
    expect(-> di.get 'TestClassA').toThrowError()
    return

  ######################################################################################################################

  it 'should be delete instance', ->
    di.set 'TestClassA', TestClassA
    expect(di.get('TestClassA')).toBe(TestClassA)
    di.delete 'TestClassA'

    expect(-> di.get 'TestClassA').toThrowError()
    return

  ######################################################################################################################

  it 'should be create object on base dependency', ->
    di.set 'TestClassA', TestClassA

    obj = di.createObject 'TestClassA', ['a', 'b', 'c']

    expect(obj instanceof TestClassA).toBeTruthy()
    expect(obj.getA()).toBe('a')
    expect(obj.getB()).toBe('b')
    expect(obj.getC()).toBe('c')
    return

  ######################################################################################################################

  it 'should be set instance', ->
    di
      .set 'TestClassA', TestClassA
      .setInstance 'TestClassA', ['a', 'b', 'c']
    obj = di.getInstance 'TestClassA'

    expect(obj instanceof TestClassA).toBeTruthy()
    expect(obj.getA()).toBe('a')
    expect(obj.getB()).toBe('b')
    expect(obj.getC()).toBe('c')
    return

  ######################################################################################################################

  it 'should be set throw error, if instance is not set', ->
    di.set 'TestClassA', TestClassA

    expect(-> di.getInstance 'TestClassA').toThrowError()
    return

  ######################################################################################################################

  it 'should be delete instance', ->
    di
      .set 'TestClassA', TestClassA
      .setInstance 'TestClassA', ['a', 'b', 'c']
    obj = di.getInstance 'TestClassA'

    expect(obj instanceof TestClassA).toBeTruthy()

    di.deleteInstance 'TestClassA'
    expect(-> di.getInstance 'TestClassA').toThrowError()
    return

  ######################################################################################################################

  it 'should be resolve if set all require dependencies', ->
    di
      .set 'TestClassA', TestClassA
      .set 'TestClassB', TestClassB
      .set 'TestClassC', TestClassC
      .setInstance 'TestClassA', ['a', 'b', 'c']
      .setInstance 'TestClassC', ['d', 'e', 'f']

    expect(-> di.resolve ['TestClassA', 'TestClassB', 'TestClassC'], ['TestClassA', 'TestClassC']).not.toThrowError()
    return

  ######################################################################################################################

  it 'should be set throw error in resolve, if set not all require dependencies', ->
    di
      .set 'TestClassA', TestClassA
      .set 'TestClassC', TestClassC
      .setInstance 'TestClassA', ['a', 'b', 'c']

    expect(-> di.resolve ['TestClassA', 'TestClassB', 'TestClassC']).toThrowError()
    expect(-> di.resolve ['TestClassA', 'TestClassC'],['TestClassC']).toThrowError()
    return
