###
# Test for events
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

DI = require '../../src/di/DI'
Events = require '../../src/events/Events'

describe 'events.Events', ->
  $ = null
  $return = null
  di = null
  ns = null
  handler = null

  beforeEach ->
    $return = jasmine.createSpyObj '$', [
      'on', 'trigger'
    ]
    $ = (obj) ->
      $return

    di = new DI()
    di.set '$', $

    handler = -> 1
    ns = 'attachments'

  ######################################################################################################################

  it 'should be exist resolved dependencies', ->
    expect(-> new Events di).not.toThrowError()
    return

  ######################################################################################################################

  it 'should be invoke $ in "on" for require object', ->
    test = jasmine.createSpyObj 'test', ['invoke']
    $ = (obj) ->
      test.invoke obj
      $return
    di.set '$', $

    obj = id: 1
    events = new Events di
    events.on obj, 'test', ->

    expect(test.invoke).toHaveBeenCalledWith obj
    return

  ######################################################################################################################

  it 'should be invoke $ in "trigger" for require object', ->
    test = jasmine.createSpyObj 'test', ['invoke']
    $ = (obj) ->
      test.invoke obj
      $return
    di.set '$', $

    obj = id: 1
    events = new Events di
    events.trigger obj, 'test'

    expect(test.invoke).toHaveBeenCalledWith obj
    return

  ######################################################################################################################

  it 'should be set on handler event for native object', ->
    obj = id: 1
    events = new Events di
    events.on obj, 'test', handler

    expect($return.on).toHaveBeenCalledWith 'test.' + ns, null, handler
    return

  ######################################################################################################################

  it 'should be set on handler event for jquery object', ->
    $obj = $ id: 1
    events = new Events di
    events.on $obj, 'test', handler

    expect($return.on).toHaveBeenCalledWith 'test.' + ns, null, handler
    return

  ######################################################################################################################

  it 'should be set on handler event for object by selector', ->
    obj = id: 1
    events = new Events di
    events.on obj, 'test', handler, selector: '.selector'

    expect($return.on).toHaveBeenCalledWith 'test.' + ns, '.selector', handler
    return

  ######################################################################################################################

  it 'should be set trigger event for object', ->
    obj = id: 1
    data = [1, 2, 3]
    events = new Events di
    events.trigger obj, 'test', data

    expect($return.trigger).toHaveBeenCalledWith 'test.' + ns, data
    return
