###
# Test for server
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

DI = require '../../src/di/DI'

Server = require '../../src/server/Server'

class TestAjax
  setSuccess: (@success) ->
    @
  setResult: (@result) ->
    @
  setJqXHR: (@jqXHR) ->
    @
  setTextStatus: (@textStatus) ->
    @
  done: (fn) ->
    fn(@result) if fn and @success
    @
  fail: (fn) ->
    fn(@jqXHR, @textStatus) if fn and @success isnt true
    @

describe 'server.Server', ->
  di = null
  $ = null
  $ajax = null
  schemes = {}
  options = {}

  beforeEach ->
    $ajax = new TestAjax()
    $ =
      ajax: -> $ajax

    options =
      schemes:
        requests:
          find:
            field: 'field'
          list:
            field: 'field'
          upload:
            field: 'field'
          delete:
            field: 'field'
        responses:
          file:
            field: 'field'
          find:
            field: 'field'
          list:
            field: 'field'
          upload:
            field: 'field'

    createScheme = (type)-> jasmine.createSpyObj(type, ['toData', 'toScheme', 'getSchemeFieldName'])
    schemes =
      requests:
        find: createScheme 'Find'
        list: createScheme 'List'
        upload: createScheme 'Upload'
        delete: createScheme 'Delete'
      responses:
        file: createScheme 'File'
        find: createScheme 'Find'
        list: createScheme 'List'
        upload: createScheme 'Upload'

    di = new DI()
    di
      .set '$', $
      .set 'server.schemes.requests.Find', -> schemes.requests.find
      .set 'server.schemes.requests.List', -> schemes.requests.list
      .set 'server.schemes.requests.Upload', -> schemes.requests.upload
      .set 'server.schemes.requests.Delete', -> schemes.requests.delete
      .set 'server.schemes.responses.File', -> schemes.responses.file
      .set 'server.schemes.responses.Find', -> schemes.responses.find
      .set 'server.schemes.responses.List', -> schemes.responses.list
      .set 'server.schemes.responses.Upload', -> schemes.responses.upload

  ######################################################################################################################

  it 'should be exist resolved dependencies', ->
    expect(-> new Server di, options).not.toThrowError()
    return

  ######################################################################################################################

  it 'should be call onSuccess in find when query is done', ->
    data = {param: 1}
    resultData = {id: 1}
    schemeResultData = {scheme_id: 1}

    test = jasmine.createSpyObj 'test', ['invoke']
    schemes.responses.find.toData.and.callFake ()->
      schemeResultData

    $ajax
      .setSuccess true
      .setResult resultData

    server = new Server di, options
    server.find '/find', data, (result) -> test.invoke result

    expect(schemes.requests.find.toScheme).toHaveBeenCalledWith data
    expect(schemes.responses.find.toData).toHaveBeenCalledWith resultData
    expect(test.invoke).toHaveBeenCalledWith schemeResultData

  ######################################################################################################################

  it 'should be call onError in find when query is fail', ->
    jqXHR = {j: 'j'}
    data = {param: 1}

    test = jasmine.createSpyObj 'test', ['invoke']
    $ajax
      .setSuccess false
      .setJqXHR jqXHR
      .setTextStatus 'failed'

    server = new Server di, options
    server.find '/find', data, null, (jqXHR, textStatus) -> test.invoke jqXHR, textStatus

    expect(schemes.requests.find.toScheme).toHaveBeenCalledWith data
    expect(test.invoke).toHaveBeenCalledWith jqXHR, 'failed'

  ######################################################################################################################

  it 'should be call onSuccess in list when query is done', ->
    data = {param: 1}
    resultData = {id: 1}
    schemeResultData = {scheme_id: 1}

    test = jasmine.createSpyObj 'test', ['invoke']
    schemes.responses.list.toData.and.callFake ()->
      schemeResultData

    $ajax
      .setSuccess true
      .setResult resultData

    server = new Server di, options
    server.list '/list', data, (result) -> test.invoke result

    expect(schemes.requests.list.toScheme).toHaveBeenCalledWith data
    expect(schemes.responses.list.toData).toHaveBeenCalledWith resultData
    expect(test.invoke).toHaveBeenCalledWith schemeResultData

  ######################################################################################################################

  it 'should be call onError in list when query is fail', ->
    jqXHR = {j: 'j'}
    data = {param: 1}

    test = jasmine.createSpyObj 'test', ['invoke']
    $ajax
      .setSuccess false
      .setJqXHR jqXHR
      .setTextStatus 'failed'

    server = new Server di, options
    server.list '/list', data, null, (jqXHR, textStatus) -> test.invoke jqXHR, textStatus

    expect(schemes.requests.list.toScheme).toHaveBeenCalledWith data
    expect(test.invoke).toHaveBeenCalledWith jqXHR, 'failed'

  ######################################################################################################################

  it 'should be call onSuccess in upload when query is done', ->
    files = [
      {file: 1}
      {file: 2}
      {file: 3}
    ]
    data = jasmine.createSpyObj 'FormData', ['getAll'];
    data.getAll.and.callFake ()->
      files

    resultData = {id: 1}
    schemeResultData = {scheme_id: 1}

    test = jasmine.createSpyObj 'test', ['invoke']
    schemes.requests.upload.getSchemeFieldName.and.callFake ()->
      'test'
    schemes.responses.upload.toData.and.callFake ()->
      schemeResultData

    $ajax
      .setSuccess true
      .setResult resultData

    server = new Server di, options
    server.upload '/upload', data, (result) -> test.invoke result

    expect(schemes.requests.upload.getSchemeFieldName).toHaveBeenCalled()
    expect(schemes.responses.upload.toData).toHaveBeenCalledWith resultData
    expect(test.invoke).toHaveBeenCalledWith schemeResultData

  ######################################################################################################################

  it 'should be call onError in upload when query is fail', ->
    jqXHR = {j: 'j'}
    files = [
      {file: 1}
      {file: 2}
      {file: 3}
    ]
    data = jasmine.createSpyObj 'FormData', ['getAll'];
    data.getAll.and.callFake ()->
      files

    test = jasmine.createSpyObj 'test', ['invoke']
    schemes.requests.upload.getSchemeFieldName.and.callFake ()->
      'test'
    $ajax
      .setSuccess false
      .setJqXHR jqXHR
      .setTextStatus 'failed'

    server = new Server di, options
    server.upload '/upload', data, null, (jqXHR, textStatus) -> test.invoke jqXHR, textStatus

    expect(schemes.requests.upload.getSchemeFieldName).toHaveBeenCalled()
    expect(test.invoke).toHaveBeenCalledWith jqXHR, 'failed'

  ######################################################################################################################

  it 'should be call onSuccess in delete when query is done', ->
    id = 1
    test = jasmine.createSpyObj 'test', ['invoke']

    $ajax
      .setSuccess true
      .setResult null

    server = new Server di, options
    server.delete '/delete', id, () -> test.invoke()

    expect(schemes.requests.delete.toScheme).toHaveBeenCalledWith id: id
    expect(test.invoke).toHaveBeenCalled()

  ######################################################################################################################

  it 'should be call onError in upload when query is fail', ->
    id = 1
    jqXHR = {j: 'j'}

    test = jasmine.createSpyObj 'test', ['invoke']
    $ajax
      .setSuccess false
      .setJqXHR jqXHR
      .setTextStatus 'failed'

    server = new Server di, options
    server.delete '/delete', id, null, (jqXHR, textStatus) -> test.invoke jqXHR, textStatus

    expect(schemes.requests.delete.toScheme).toHaveBeenCalledWith id: id
    expect(test.invoke).toHaveBeenCalledWith jqXHR, 'failed'
