###
# Test for DataMap
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

DataMap = require '../../src/utils/DataMap'

describe 'utils.DataMap', ->

  it 'should be add items in the constructor', ->
    map = new DataMap [1..3]

    items = map.all()
    values = [items[0].value, items[1].value, items[2].value]

    expect(values.indexOf(1) isnt -1).toBeTruthy()
    expect(values.indexOf(2) isnt -1).toBeTruthy()
    expect(values.indexOf(3) isnt -1).toBeTruthy()
    return

  ######################################################################################################################

  it 'should be exist if item add', ->
    map = new DataMap

    map.add 'key1', 'value1'
    map.add 'key2', 'value2'
    map.add 'key3', 'value3'
    expect(map.has('key1')).toBeTruthy()
    expect(map.has('key2')).toBeTruthy()
    expect(map.has('key3')).toBeTruthy()
    return

  ######################################################################################################################

  it 'should be not exist if item not added', ->
    map = new DataMap

    map.add 'key4', 'value4'
    map.add 'key5', 'value5'

    expect(map.has('key1')).toBeFalsy()
    expect(map.has('key2')).toBeFalsy()
    expect(map.has('key3')).toBeFalsy()
    return

  ######################################################################################################################

  it 'should be add items in the addItems', ->
    map = new DataMap [1..3]

    items = map.all()
    values = [items[0].value, items[1].value, items[2].value]

    expect(values.indexOf(1) isnt -1).toBeTruthy()
    expect(values.indexOf(2) isnt -1).toBeTruthy()
    expect(values.indexOf(3) isnt -1).toBeTruthy()
    return

  ######################################################################################################################

  it 'should be add item in the add method ', ->
    map = new DataMap
    map.add 0, 1
    map.add 1, 2
    map.add 2, 3

    items = map.all()
    values = [items[0].value, items[1].value, items[2].value]

    expect(values.indexOf(1) isnt -1).toBeTruthy()
    expect(values.indexOf(2) isnt -1).toBeTruthy()
    expect(values.indexOf(3) isnt -1).toBeTruthy()
    return

  ######################################################################################################################

  it 'should be get returned item by key with defined value', ->
    map = new DataMap

    map.add 'key1', 'value1'
    map.add 'key2', 1234
    map.add 'key3', [1..3]

    expect(map.get('key1')).toEqual('value1')
    expect(map.get('key2')).toEqual(1234)
    expect(map.get('key3')).toEqual([1..3])
    return

  ######################################################################################################################

  it 'should be remove item by key', ->
    map = new DataMap

    map.add 'key1', 'value1'
    map.add 'key2', 'value2'
    map.add 'key3', 'value3'

    map.remove 'key1'
    map.remove 'key2'
    map.remove 'key3'

    expect(map.get('key1')).toBeNull()
    expect(map.get('key2')).toBeNull()
    expect(map.get('key3')).toBeNull()
    return

  ######################################################################################################################

  it 'should be invoke function for each item', ->
    map = new DataMap
    map.add 'key1', 'value1'
    map.add 'key2', 'value2'
    map.add 'key3', 'value3'

    args = {}
    fn = (key, value)->
      args[key] = value
    map.each fn

    expect(args['key1']).toEqual('value1')
    expect(args['key2']).toEqual('value2')
    expect(args['key3']).toEqual('value3')
    return

  ######################################################################################################################

  it 'should be return all items', ->
    map = new DataMap

    map.add 'key1', 'value1'
    map.add 'key2', 'value2'
    map.add 'key3', 'value3'

    items = map.all()
    values = [items[0].value, items[1].value, items[2].value]

    expect(values.indexOf('value1') isnt -1).toBeTruthy()
    expect(values.indexOf('value2') isnt -1).toBeTruthy()
    expect(values.indexOf('value3') isnt -1).toBeTruthy()
    return

  ######################################################################################################################

  it 'should return all items in the order of adding an them', ->
    map = new DataMap

    map.add 'key1', 'value1'
    map.add 'key2', 'value2'
    map.add 'key3', 'value3'

    items = map.all()

    expect(items[0].value).toEqual('value1')
    expect(items[1].value).toEqual('value2')
    expect(items[2].value).toEqual('value3')
    return

  ######################################################################################################################

  it 'should return array with all items in the order of adding them', ->
    map = new DataMap

    map.add 'key2', 'value2'
    map.add 'key3', 'value3'
    map.add 'key1', 'value1'

    items = map.toArray()

    expect(items[0]).toEqual('value2')
    expect(items[1]).toEqual('value3')
    expect(items[2]).toEqual('value1')
    return

  ######################################################################################################################

  it 'should be remove all items in clear', ->

    map = new DataMap

    map.add 'key1', 'value1'
    map.add 'key2', 'value2'
    map.add 'key3', 'value3'

    map.clear()

    expect(map.length()).toEqual(0)
    return

  ######################################################################################################################

  it 'should be return corrected count added items', ->
    map = new DataMap

    map.add 'key1', 'value1'
    map.add 'key2', 'value2'
    map.add 'key3', 'value3'

    expect(map.length()).toEqual(3)
    return

  ######################################################################################################################

  it 'should be return corrected count added items after remove', ->
    map = new DataMap

    map.add 'key1', 'value1'
    map.add 'key2', 'value2'
    map.add 'key3', 'value3'
    map.add 'key4', 'value4'
    map.add 'key5', 'value5'
    map.remove 'key3'

    expect(map.length()).toEqual(4)
    return
